package com.dayou.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dayou.entity.ReportCalculateParam;
import com.dayou.entity.ReportCalculateTable;
import com.dayou.entity.ReportSection;
import com.dayou.mapper.ReportSectionMapper;
import com.dayou.report.ActionConvertHandler;
import com.dayou.report.calculation.cityPrice.annotation.TableHead;
import com.dayou.report.calculation.cityPrice.process.CityPriceCalculateProcess;
import com.dayou.report.dto.CalculateDataRequestDTO;
import com.dayou.report.dto.InputData;
import com.dayou.report.dto.InputTableData;
import com.dayou.report.dto.Params;
import com.dayou.service.IReportCalculateParamService;
import com.dayou.service.IReportCalculateTableService;
import com.dayou.service.IReportMainService;
import com.dayou.service.IReportSectionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.utils.ConvertUtil;
import com.dayou.vo.ReportSectionTreeVO;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * 报告段落 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2023-08-29
 */
@Service
public class ReportSectionServiceImpl extends ServiceImpl<ReportSectionMapper, ReportSection> implements IReportSectionService {

    @Autowired
    private ActionConvertHandler actionConvertHandler;

    @Autowired
    private IReportCalculateParamService reportCalculateParamService;

    @Autowired
    private IReportCalculateTableService reportCalculateTableService;



    @Override
    @SuppressWarnings("unchecked")
    public Page<ReportSection> selectPage(Page page,ReportSection reportSection){
        return this.page(page, new QueryWrapper<ReportSection>(reportSection));
    }


    @Override
    public ReportSection detail(Long id){
        return this.getById(id);
    }

    @Override
    public Boolean add(ReportSection reportSection){
        return  this.save(reportSection);
    }

    @Override
    public Boolean update(ReportSection reportSection){
        return  this.updateById(reportSection);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Override
    public Boolean updateSection(ReportSectionTreeVO treeVO) {
        ReportSection reportSection = new ReportSection();
        reportSection.setId(treeVO.getSectionId());
        reportSection.setSection(treeVO.getSection());
        return this.updateById(reportSection);
    }

    @Override
    public Boolean calculate(CalculateDataRequestDTO calculateData) {
        String action = calculateData.getAction();
        switch (action) {
            case "businessCenter":
               return actionConvertHandler.businessCenter54(calculateData.getJsonString());

        }
        return false;
    }

    @Override
    public InputData inputDataStructure(Long id) {
        List<ReportCalculateParam> params = reportCalculateParamService.list(new LambdaQueryWrapper<ReportCalculateParam>().eq(ReportCalculateParam::getHeadlineId, id));
        List<ReportCalculateTable> tables = reportCalculateTableService.list(new LambdaQueryWrapper<ReportCalculateTable>().eq(ReportCalculateTable::getHeadlineId, id));
        List<Params> xParams = ConvertUtil.copyList(params, Params.class);
        List<InputTableData> xTables = Lists.newArrayList();
        try {
            for (ReportCalculateTable table : tables){
                InputTableData inputTableData = new InputTableData();
                ConvertUtil.copyProperties(table,inputTableData);
                String tableClass = table.getTableClass();
                List<Field> fields = Arrays.asList(Class.forName(tableClass).getDeclaredFields());
                List<com.dayou.report.dto.TableHead> heads = Lists.newArrayList();
                for (Field field : fields){
                    field.setAccessible(true);
                    TableHead tableHead = field.getAnnotation(TableHead.class);
                    if (tableHead.isRequest()) {
                        com.dayou.report.dto.TableHead th = new com.dayou.report.dto.TableHead();
                        String label = tableHead.name();
                        String prop = field.getName();
                        th.setLabel(label);
                        th.setProp(prop);
                        heads.add(th);
                    }
                }
                inputTableData.setTableHead(heads);
                xTables.add(inputTableData);
            }
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
        InputData structure = InputData.builder().inputTableData(xTables).params(xParams).build();
        return structure;
    }

}
